@@ -23,7 +23,7 @@
  */
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/addrtoname.c,v 1.96.2.3 2003/12/15 04:02:53 guy Exp $ (LBL)";
+    "@(#) $Header: /tcpdump/master/tcpdump/addrtoname.c,v 1.104 2004/03/24 04:13:38 guy Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -103,6 +103,10 @@ win32_gethostbyaddr(const char *addr, int len, int type)
 		memset(&addr6, 0, sizeof(addr6));
 		addr6.sin6_family = AF_INET6;
 		memcpy(&addr6.sin6_addr, addr, len);
+#ifdef __MINGW32__
+		/* MinGW doesn't provide getnameinfo */
+		return NULL;
+#else
 		if (getnameinfo((struct sockaddr *)&addr6, sizeof(addr6),
 			hname, sizeof(hname), NULL, 0, 0)) {
 		    return NULL;
@@ -110,6 +114,7 @@ win32_gethostbyaddr(const char *addr, int len, int type)
 			strcpy(host.h_name, hname);
 			return &host;
 		}
+#endif /* __MINGW32__ */
 		break;
 	default:
 		return NULL;
@@ -190,6 +195,21 @@ static u_int32_t f_localnet;
 /*
  * Return a name for the IP address pointed to by ap.  This address
  * is assumed to be in network byte order.
+ *
+ * NOTE: ap is *NOT* necessarily part of the packet data (not even if
+ * this is being called with the "ipaddr_string()" macro), so you
+ * *CANNOT* use the TCHECK{2}/TTEST{2} macros on it.  Furthermore,
+ * even in cases where it *is* part of the packet data, the caller
+ * would still have to check for a null return value, even if it's
+ * just printing the return value with "%s" - not all versions of
+ * printf print "(null)" with "%s" and a null pointer, some of them
+ * don't check for a null pointer and crash in that case.
+ *
+ * The callers of this routine should, before handing this routine
+ * a pointer to packet data, be sure that the data is present in
+ * the packet buffer.  They should probably do those checks anyway,
+ * as other data at that layer might not be IP addresses, and it
+ * also needs to check whether they're present in the packet buffer.
  */
 const char *
 getname(const u_char *ap)
@@ -466,7 +486,7 @@ etheraddr_string(register const u_char *ep)
 const char *
 linkaddr_string(const u_char *ep, const unsigned int len)
 {
-	register u_int i, j;
+	register u_int i;
 	register char *cp;
 	register struct enamemem *tp;
 
@@ -480,13 +500,11 @@ linkaddr_string(const u_char *ep, const unsigned int len)
 	tp->e_name = cp = (char *)malloc(len*3);
 	if (tp->e_name == NULL)
 		error("linkaddr_string: malloc");
-	if ((j = *ep >> 4) != 0)
-		*cp++ = hex[j];
+	*cp++ = hex[*ep >> 4];
 	*cp++ = hex[*ep++ & 0xf];
 	for (i = len-1; i > 0 ; --i) {
 		*cp++ = ':';
-		if ((j = *ep >> 4) != 0)
-			*cp++ = hex[j];
+		*cp++ = hex[*ep >> 4];
 		*cp++ = hex[*ep++ & 0xf];
 	}
 	*cp = '\0';
