@@ -1,4 +1,4 @@
-/* @(#) $Header: /tcpdump/master/tcpdump/ospf.h,v 1.3 2000/12/17 23:07:50 guy Exp $ (LBL) */
+/* @(#) $Header: /tcpdump/master/tcpdump/ospf.h,v 1.13 2004/01/27 13:33:24 hannes Exp $ (LBL) */
 /*
  * Copyright (c) 1991, 1993, 1994, 1995, 1996, 1997
  *	The Regents of the University of California.  All rights reserved.
@@ -21,25 +21,34 @@
  *
  * OSPF support contributed by Jeffrey Honig (jch@mitchell.cit.cornell.edu)
  */
-#define	OSPF_TYPE_UMD	0	/* UMd's special monitoring packets */
-#define	OSPF_TYPE_HELLO	1	/* Hello */
-#define	OSPF_TYPE_DB	2	/* Database Description */
-#define	OSPF_TYPE_LSR	3	/* Link State Request */
-#define	OSPF_TYPE_LSU	4	/* Link State Update */
-#define	OSPF_TYPE_LSA	5	/* Link State Ack */
-#define	OSPF_TYPE_MAX	6
+#define	OSPF_TYPE_UMD           0	/* UMd's special monitoring packets */
+#define	OSPF_TYPE_HELLO         1	/* Hello */
+#define	OSPF_TYPE_DD            2	/* Database Description */
+#define	OSPF_TYPE_LS_REQ        3	/* Link State Request */
+#define	OSPF_TYPE_LS_UPDATE     4	/* Link State Update */
+#define	OSPF_TYPE_LS_ACK        5	/* Link State Ack */
 
-/* Options *_options	*/
+/* Options field
+ *
+ * +------------------------------------+
+ * | * | O | DC | EA | N/P | MC | E | T |
+ * +------------------------------------+
+ *
+ */
+                
 #define OSPF_OPTION_T	0x01	/* T bit: TOS support	*/
 #define OSPF_OPTION_E	0x02	/* E bit: External routes advertised	*/
 #define	OSPF_OPTION_MC	0x04	/* MC bit: Multicast capable */
+#define	OSPF_OPTION_NP	0x08	/* N/P bit: NSSA capable */
+#define	OSPF_OPTION_EA	0x10	/* EA bit: External Attribute capable */
+#define	OSPF_OPTION_DC	0x20	/* DC bit: Demand circuit capable */
+#define	OSPF_OPTION_O	0x40	/* O bit: Opaque LSA capable */
 
 /* ospf_authtype	*/
 #define	OSPF_AUTH_NONE		0	/* No auth-data */
 #define	OSPF_AUTH_SIMPLE	1	/* Simple password */
 #define OSPF_AUTH_MD5		2	/* MD5 authentication */
-#define     OSPF_AUTH_MD5_LEN	16	/* length of MD5 authentication */
-
+#define OSPF_AUTH_MD5_LEN	16	/* length of MD5 authentication */
 
 /* db_flags	*/
 #define	OSPF_DB_INIT		0x04	    /*	*/
@@ -54,7 +63,43 @@
 #define	LS_TYPE_ASE		5   /* ASE  */
 #define	LS_TYPE_GROUP		6   /* Group membership (multicast */
 				    /* extensions 23 July 1991) */
-#define	LS_TYPE_MAX		7
+#define	LS_TYPE_NSSA            7   /* rfc1587 - Not so Stubby Areas */
+#define	LS_TYPE_OPAQUE_LL       9   /* rfc2370 - Opaque Link Local */
+#define	LS_TYPE_OPAQUE_AL      10   /* rfc2370 - Opaque Link Local */
+#define	LS_TYPE_OPAQUE_DW      11   /* rfc2370 - Opaque Domain Wide */
+
+#define LS_OPAQUE_TYPE_TE       1   /* rfc3630 */
+#define LS_OPAQUE_TYPE_GRACE    3   /* rfc3623 */
+
+#define LS_OPAQUE_TE_TLV_ROUTER 1   /* rfc3630 */
+#define LS_OPAQUE_TE_TLV_LINK   2   /* rfc3630 */
+
+#define LS_OPAQUE_TE_LINK_SUBTLV_LINK_TYPE             1 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_LINK_ID               2 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_LOCAL_IP              3 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_REMOTE_IP             4 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_TE_METRIC             5 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_MAX_BW                6 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_MAX_RES_BW            7 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_UNRES_BW              8 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_ADMIN_GROUP           9 /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_LINK_LOCAL_REMOTE_ID 11 /* draft-ietf-ccamp-ospf-gmpls-extensions */
+#define LS_OPAQUE_TE_LINK_SUBTLV_LINK_PROTECTION_TYPE 14 /* draft-ietf-ccamp-ospf-gmpls-extensions */
+#define LS_OPAQUE_TE_LINK_SUBTLV_INTF_SW_CAP_DESCR    15 /* draft-ietf-ccamp-ospf-gmpls-extensions */
+#define LS_OPAQUE_TE_LINK_SUBTLV_SHARED_RISK_GROUP    16 /* draft-ietf-ccamp-ospf-gmpls-extensions */
+#define LS_OPAQUE_TE_LINK_SUBTLV_DIFFSERV_TE          17 /* draft-ietf-tewg-diff-te-proto-06 */
+
+#define LS_OPAQUE_TE_LINK_SUBTLV_LINK_TYPE_PTP        1  /* rfc3630 */
+#define LS_OPAQUE_TE_LINK_SUBTLV_LINK_TYPE_MA         2  /* rfc3630 */
+
+#define LS_OPAQUE_GRACE_TLV_PERIOD       1 /* rfc3623 */
+#define LS_OPAQUE_GRACE_TLV_REASON       2 /* rfc3623 */
+#define LS_OPAQUE_GRACE_TLV_INT_ADDRESS  3 /* rfc3623 */
+
+#define LS_OPAQUE_GRACE_TLV_REASON_UNKNOWN     0 /* rfc3623 */
+#define LS_OPAQUE_GRACE_TLV_REASON_SW_RESTART  1 /* rfc3623 */
+#define LS_OPAQUE_GRACE_TLV_REASON_SW_UPGRADE  2 /* rfc3623 */
+#define LS_OPAQUE_GRACE_TLV_REASON_CP_SWITCH   3 /* rfc3623 */
 
 /*************************************************
  *
@@ -95,12 +140,18 @@ struct lsa_hdr {
     u_int16_t ls_age;
     u_int8_t ls_options;
     u_int8_t ls_type;
-    struct in_addr ls_stateid;
+    union {
+        struct in_addr lsa_id;
+        struct { /* opaque LSAs change the LSA-ID field */
+            u_int8_t opaque_type;
+            u_int8_t opaque_id[3];
+	} opaque_field;
+    } un_lsa_id;
     struct in_addr ls_router;
     u_int32_t ls_seq;
     u_int16_t ls_chksum;
     u_int16_t ls_length;
-} ;
+};
 
 /* link state advertisement */
 struct lsa {
@@ -149,8 +200,28 @@ struct lsa {
 	    u_int32_t mcla_vtype;
 	    struct in_addr mcla_vid;
 	} un_mcla[1];
+
+        /* Opaque TE LSA */
+        struct {
+	    u_int16_t type;
+	    u_int16_t length;
+	    u_int8_t data[1]; /* may repeat   */
+	} un_te_lsa_tlv;
+
+        /* Grace LSA */
+        struct {
+	    u_int16_t type;
+	    u_int16_t length;
+	    u_int8_t data[1]; /* may repeat   */
+	} un_grace_tlv;
+
+        /* Unknown LSA */
+        struct unknown {
+	    u_int8_t data[1]; /* may repeat   */
+	} un_unknown[1];
+
     } lsa_un;
-} ;
+};
 
 
 /*
@@ -160,7 +231,7 @@ struct tos_metric {
     u_int8_t tos_type;
     u_int8_t tos_zero;
     u_int16_t tos_metric;
-} ;
+};
 
 #define	OSPF_AUTH_SIZE	8
 
@@ -201,8 +272,14 @@ struct ospfhdr {
 
 	/* Link State Request */
 	struct lsr {
-	    u_int32_t ls_type;
-	    struct in_addr ls_stateid;
+	    u_int8_t ls_type[4];
+            union {
+                struct in_addr ls_stateid;
+                struct { /* opaque LSAs change the LSA-ID field */
+                    u_int8_t opaque_type;
+                    u_int8_t opaque_id[3];
+                } opaque_field;
+            } un_ls_stateid;
 	    struct in_addr ls_router;
 	} un_lsr[1];		/* may repeat	*/
 
@@ -217,7 +294,7 @@ struct ospfhdr {
 	    struct lsa_hdr lsa_lshdr[1]; /* may repeat	*/
 	} un_lsa ;
     } ospf_un ;
-} ;
+};
 
 #define	ospf_hello	ospf_un.un_hello
 #define	ospf_db		ospf_un.un_db
