@@ -33,29 +33,25 @@
 /* Cisco NetFlow protocol */
 
 #ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-cnfp.c,v 1.8 2001/09/17 21:57:58 fenner Exp $";
+static const char rcsid[] _U_ =
+    "@(#) $Header: /tcpdump/master/tcpdump/print-cnfp.c,v 1.16 2003/11/16 09:36:16 guy Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <sys/types.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-#include <netdb.h>
-
-#include <netinet/in.h>
-
-#include <arpa/inet.h>
+#include <tcpdump-stdinc.h>
 
 #include <stdio.h>
 #include <string.h>
 
 #include "interface.h"
+#include "addrtoname.h"
+#include "extract.h"
 
 #include "tcp.h"
+#include "ipproto.h"
 
 struct nfhdr {
 	u_int32_t	ver_cnt;	/* version [15], and # of records */
@@ -83,7 +79,7 @@ struct nfrec {
 };
 
 void
-cnfp_print(const u_char *cp, u_int len, const u_char *bp)
+cnfp_print(const u_char *cp, const u_char *bp)
 {
 	register const struct nfhdr *nh;
 	register const struct nfrec *nr;
@@ -98,18 +94,18 @@ cnfp_print(const u_char *cp, u_int len, const u_char *bp)
 	if ((const u_char *)(nh + 1) > snapend)
 		return;
 
-	nrecs = ntohl(nh->ver_cnt) & 0xffff;
-	ver = (ntohl(nh->ver_cnt) & 0xffff0000) >> 16;
-	t = ntohl(nh->utc_sec);
+	nrecs = EXTRACT_32BITS(&nh->ver_cnt) & 0xffff;
+	ver = (EXTRACT_32BITS(&nh->ver_cnt) & 0xffff0000) >> 16;
+	t = EXTRACT_32BITS(&nh->utc_sec);
 /*	(p = ctime(&t))[24] = '\0'; */
 
 	printf("NetFlow v%x, %u.%03u uptime, %u.%09u, ", ver,
-	       (unsigned)ntohl(nh->msys_uptime)/1000,
-	       (unsigned)ntohl(nh->msys_uptime)%1000,
-	       (unsigned)ntohl(nh->utc_sec), (unsigned)ntohl(nh->utc_nsec));
+	       EXTRACT_32BITS(&nh->msys_uptime)/1000,
+	       EXTRACT_32BITS(&nh->msys_uptime)%1000,
+	       EXTRACT_32BITS(&nh->utc_sec), EXTRACT_32BITS(&nh->utc_nsec));
 
 	if (ver == 5 || ver == 6) {
-		printf("#%u, ", (unsigned)htonl(nh->sequence));
+		printf("#%u, ", EXTRACT_32BITS(&nh->sequence));
 		nr = (const struct nfrec *)&nh[1];
 		snaplen -= 24;
 	} else {
@@ -124,36 +120,36 @@ cnfp_print(const u_char *cp, u_int len, const u_char *bp)
 		char asbuf[20];
 
 		printf("\n  started %u.%03u, last %u.%03u",
-		       (unsigned)ntohl(nr->start_time)/1000,
-		       (unsigned)ntohl(nr->start_time)%1000,
-		       (unsigned)ntohl(nr->last_time)/1000,
-		       (unsigned)ntohl(nr->last_time)%1000);
+		       EXTRACT_32BITS(&nr->start_time)/1000,
+		       EXTRACT_32BITS(&nr->start_time)%1000,
+		       EXTRACT_32BITS(&nr->last_time)/1000,
+		       EXTRACT_32BITS(&nr->last_time)%1000);
 
 		asbuf[0] = buf[0] = '\0';
 		if (ver == 5 || ver == 6) {
 			snprintf(buf, sizeof(buf), "/%u",
-				 (unsigned)(ntohl(nr->masks) >> 24) & 0xff);
+				 (EXTRACT_32BITS(&nr->masks) >> 24) & 0xff);
 			snprintf(asbuf, sizeof(asbuf), ":%u",
-				 (unsigned)(ntohl(nr->asses) >> 16) & 0xffff);
+				 (EXTRACT_32BITS(&nr->asses) >> 16) & 0xffff);
 		}
-		printf("\n    %s%s%s:%u ", inet_ntoa(nr->src_ina), buf, asbuf,
-			(unsigned)ntohl(nr->ports) >> 16);
+		printf("\n    %s%s%s:%u ", intoa(nr->src_ina.s_addr), buf, asbuf,
+			EXTRACT_32BITS(&nr->ports) >> 16);
 
 		if (ver == 5 || ver ==6) {
 			snprintf(buf, sizeof(buf), "/%d",
-				 (unsigned)(ntohl(nr->masks) >> 16) & 0xff);
+				 (EXTRACT_32BITS(&nr->masks) >> 16) & 0xff);
 			snprintf(asbuf, sizeof(asbuf), ":%u",
-				 (unsigned)ntohl(nr->asses) & 0xffff);
+				 EXTRACT_32BITS(&nr->asses) & 0xffff);
 		}
-		printf("> %s%s%s:%u ", inet_ntoa(nr->dst_ina), buf, asbuf,
-			(unsigned)ntohl(nr->ports) & 0xffff);
+		printf("> %s%s%s:%u ", intoa(nr->dst_ina.s_addr), buf, asbuf,
+			EXTRACT_32BITS(&nr->ports) & 0xffff);
 
-		printf(">> %s\n    ", inet_ntoa(nr->nhop_ina));
+		printf(">> %s\n    ", intoa(nr->nhop_ina.s_addr));
 
-		pent = getprotobynumber((ntohl(nr->proto_tos) >> 8) & 0xff);
+		pent = getprotobynumber((EXTRACT_32BITS(&nr->proto_tos) >> 8) & 0xff);
 		if (!pent || nflag)
 			printf("%u ",
-			       (unsigned)(ntohl(nr->proto_tos) >> 8) & 0xff);
+			       (EXTRACT_32BITS(&nr->proto_tos) >> 8) & 0xff);
 		else
 			printf("%s ", pent->p_name);
 
@@ -161,9 +157,9 @@ cnfp_print(const u_char *cp, u_int len, const u_char *bp)
 		if (pent && pent->p_proto == IPPROTO_TCP) {
 			int flags;
 			if (ver == 1)
-				flags = (ntohl(nr->asses) >> 24) & 0xff;
+				flags = (EXTRACT_32BITS(&nr->asses) >> 24) & 0xff;
 			else
-				flags = (ntohl(nr->proto_tos) >> 16) & 0xff;
+				flags = (EXTRACT_32BITS(&nr->proto_tos) >> 16) & 0xff;
 			if (flags & TH_FIN)	putchar('F');
 			if (flags & TH_SYN)	putchar('S');
 			if (flags & TH_RST)	putchar('R');
@@ -177,12 +173,12 @@ cnfp_print(const u_char *cp, u_int len, const u_char *bp)
 		buf[0]='\0';
 		if (ver == 6) {
 			snprintf(buf, sizeof(buf), "(%u<>%u encaps)",
-				 (unsigned)(ntohl(nr->masks) >> 8) & 0xff,
-				 (unsigned)(ntohl(nr->masks)) & 0xff);
+				 (EXTRACT_32BITS(&nr->masks) >> 8) & 0xff,
+				 (EXTRACT_32BITS(&nr->masks)) & 0xff);
 		}
 		printf("tos %u, %u (%u octets) %s",
-		       (unsigned)ntohl(nr->proto_tos) & 0xff,
-		       (unsigned)ntohl(nr->packets),
-		       (unsigned)ntohl(nr->octets), buf);
+		       EXTRACT_32BITS(&nr->proto_tos) & 0xff,
+		       EXTRACT_32BITS(&nr->packets),
+		       EXTRACT_32BITS(&nr->octets), buf);
 	}
 }
