@@ -20,8 +20,8 @@
  */
 
 #ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-rt6.c,v 1.17 2000/12/13 07:57:05 itojun Exp $";
+static const char rcsid[] _U_ =
+    "@(#) $Header: /tcpdump/master/tcpdump/print-rt6.c,v 1.26 2003/11/19 00:36:08 guy Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -30,13 +30,7 @@ static const char rcsid[] =
 
 #ifdef INET6
 
-#include <sys/param.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-
-
-#include <netinet/in.h>
+#include <tcpdump-stdinc.h>
 
 #include <stdio.h>
 
@@ -44,6 +38,7 @@ static const char rcsid[] =
 
 #include "interface.h"
 #include "addrtoname.h"
+#include "extract.h"
 
 int
 rt6_print(register const u_char *bp, register const u_char *bp2)
@@ -62,12 +57,6 @@ rt6_print(register const u_char *bp, register const u_char *bp2)
 	/* 'ep' points to the end of available data. */
 	ep = snapend;
 
-#if 0
-	printf("%s > %s: ",
-	       ip6addr_string(&ip->ip6_src),
-	       ip6addr_string(&ip->ip6_dst));
-#endif
-
 	TCHECK(dp->ip6r_segleft);
 
 	printf("srcrt (len=%d", dp->ip6r_len);	/*)*/
@@ -77,14 +66,18 @@ rt6_print(register const u_char *bp, register const u_char *bp2)
 	switch (dp->ip6r_type) {
 #ifndef IPV6_RTHDR_TYPE_0
 #define IPV6_RTHDR_TYPE_0 0
+#endif
+#ifndef IPV6_RTHDR_TYPE_2
+#define IPV6_RTHDR_TYPE_2 2
 #endif
 	case IPV6_RTHDR_TYPE_0:
+	case IPV6_RTHDR_TYPE_2:			/* Mobile IPv6 ID-20 */
 		dp0 = (struct ip6_rthdr0 *)dp;
 
 		TCHECK(dp0->ip6r0_reserved);
 		if (dp0->ip6r0_reserved || vflag) {
 			printf(", rsv=0x%0x",
-			    (u_int32_t)ntohl(dp0->ip6r0_reserved));
+			    EXTRACT_32BITS(&dp0->ip6r0_reserved));
 		}
 
 		if (len % 2 == 1)
@@ -94,7 +87,7 @@ rt6_print(register const u_char *bp, register const u_char *bp2)
 		for (i = 0; i < len; i++) {
 			if ((u_char *)(addr + 1) > ep)
 				goto trunc;
-		
+
 			printf(", [%d]%s", i, ip6addr_string(addr));
 			addr++;
 		}
@@ -109,6 +102,6 @@ rt6_print(register const u_char *bp, register const u_char *bp2)
 
  trunc:
 	fputs("[|srcrt]", stdout);
-	return 65535;		/* XXX */
+	return -1;
 }
 #endif /* INET6 */
