@@ -20,21 +20,16 @@
  */
 
 #ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-sl.c,v 1.57 2001/07/05 18:54:17 guy Exp $ (LBL)";
+static const char rcsid[] _U_ =
+    "@(#) $Header: /tcpdump/master/tcpdump/print-sl.c,v 1.64 2003/11/16 09:36:36 guy Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <sys/param.h>
-#include <sys/time.h>
+#include <tcpdump-stdinc.h>
 
-#include <netinet/in.h>
-
-#include <ctype.h>
-#include <netdb.h>
 #include <pcap.h>
 #include <stdio.h>
 
@@ -53,27 +48,17 @@ static u_int lastconn = 255;
 static void sliplink_print(const u_char *, const struct ip *, u_int);
 static void compressed_sl_print(const u_char *, const struct ip *, u_int, int);
 
-void
-sl_if_print(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
+u_int
+sl_if_print(const struct pcap_pkthdr *h, const u_char *p)
 {
 	register u_int caplen = h->caplen;
 	register u_int length = h->len;
 	register const struct ip *ip;
 
-	++infodelay;
-	ts_print(&h->ts);
-
 	if (caplen < SLIP_HDRLEN) {
 		printf("[|slip]");
-		goto out;
+		return (caplen);
 	}
-	/*
-	 * Some printers want to get back at the link level addresses,
-	 * and/or check that they're not walking off the end of the packet.
-	 * Rather than pass them all the way down, we set these globals.
-	 */
-	packetp = p;
-	snapend = p + caplen;
 
 	length -= SLIP_HDRLEN;
 
@@ -95,37 +80,20 @@ sl_if_print(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
 		printf ("ip v%d", IP_V(ip));
 	}
 
-	if (xflag)
-		default_print((u_char *)ip, caplen - SLIP_HDRLEN);
- out:
-	putchar('\n');
-	--infodelay;
-	if (infoprint)
-		info(0);
+	return (SLIP_HDRLEN);
 }
 
-
-void
-sl_bsdos_if_print(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
+u_int
+sl_bsdos_if_print(const struct pcap_pkthdr *h, const u_char *p)
 {
 	register u_int caplen = h->caplen;
 	register u_int length = h->len;
 	register const struct ip *ip;
 
-	++infodelay;
-	ts_print(&h->ts);
-
 	if (caplen < SLIP_HDRLEN) {
 		printf("[|slip]");
-		goto out;
+		return (caplen);
 	}
-	/*
-	 * Some printers want to get back at the link level addresses,
-	 * and/or check that they're not walking off the end of the packet.
-	 * Rather than pass them all the way down, we set these globals.
-	 */
-	packetp = p;
-	snapend = p + caplen;
 
 	length -= SLIP_HDRLEN;
 
@@ -138,13 +106,7 @@ sl_bsdos_if_print(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
 
 	ip_print((u_char *)ip, length);
 
-	if (xflag)
-		default_print((u_char *)ip, caplen - SLIP_HDRLEN);
- out:
-	putchar('\n');
-	--infodelay;
-	if (infoprint)
-		info(0);
+	return (SLIP_HDRLEN);
 }
 
 static void
